<!DOCTYPE html>
<html>
<head>
	<meta charset="utf-8" />
	<title>{{title}}</title>
	<link rel="edit" href="{{ editurl }}" />
	<link rel="source" href="{{ sourceurl }}" />
	<link rel="stylesheet" type="text/css" href="../styles.css" />
	<script src ="../page.js"></script>
	<script>
		var padmeta = {{ metadata_json }};
	</script>
</head>
<body>
{{body}}
</body>
</html>
